@@ -544,81 +544,3 @@ bonobo_client_site_new_item (BonoboClientSite *client_site, GnomeCanvasGroup *gr
 	
 	return item;
 }
-
-/**
- * bonobo_client_site_get_verbs:
- * @server_object: The pointer to the embedeed server object
- *
- * Returns: A GList containing the GnomeVerb structures for the verbs
- * supported by the Embeddable @server_object.
- *
- * The GnomeVerbs can be deallocated with a call to
- * bonobo_embeddable_free_verbs ().
- */
-GList *
-bonobo_client_site_get_verbs (BonoboClientSite *client_site)
-{
-	Bonobo_Embeddable_verb_list *list;
-	CORBA_Environment ev;
-	CORBA_Object server_object;
-	GList *l;
-	int i;
-	
-	g_return_val_if_fail (client_site != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
-	g_return_val_if_fail (client_site->bound_embeddable != NULL, NULL);
-
-	CORBA_exception_init (&ev);
-
-	server_object = bonobo_object_corba_objref (
-		BONOBO_OBJECT (client_site->bound_embeddable));
-	if (server_object == CORBA_OBJECT_NIL)
-		return NULL;
-
-	list = Bonobo_Embeddable_get_verb_list (server_object, &ev);
-
-	if (ev._major != CORBA_NO_EXCEPTION){
-		bonobo_object_check_env (BONOBO_OBJECT (client_site), server_object, &ev);
-		CORBA_exception_free (&ev);
-		return NULL;
-	}
-
-	for (l = NULL, i = 0; i < list->_length; i++) {
-		GnomeVerb *verb;
-
-		verb = g_new0 (GnomeVerb, 1);
-		verb->name = g_strdup (list->_buffer [i].name);
-		verb->label = g_strdup (list->_buffer [i].label);
-		verb->hint = g_strdup (list->_buffer [i].hint);
-
-		l = g_list_prepend (l, verb);
-	}
-
-	CORBA_exception_free (&ev);
-	CORBA_free (list);
-
-	return l;
-}
-
-/**
- * bonobo_client_site_free_verbs:
- */
-void
-bonobo_client_site_free_verbs (GList *verbs)
-{
-	GList *curr;
-
-	if (verbs == NULL)
-		return;
-
-	for (curr = verbs; curr != NULL; curr = curr->next) {
-		GnomeVerb *verb = (GnomeVerb *) curr->data;
-
-		g_free (verb->name);
-		g_free (verb->label);
-		g_free (verb->hint);
-		g_free (verb);
-	}
-	
-	g_list_free (verbs);
-}
