@@ -19,11 +19,11 @@
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdktypes.h>
+#include <gtk/gtkhbox.h>
 #include <bonobo/bonobo-socket.h>
 
 enum {
 	ACTIVATED,
-	UNDO_LAST_OPERATION,
 	ACTIVATE_URI,
 	LAST_SIGNAL
 };
@@ -40,7 +40,7 @@ struct _BonoboControlFramePrivate {
 	Bonobo_Control	   control;
 	GtkWidget         *container;
 	GtkWidget	  *socket;
-	Bonobo_UIContainer uih;
+	Bonobo_UIContainer ui_container;
 	BonoboPropertyBag *propbag;
 	gboolean           autoactivate;
 	gboolean           autostate;
@@ -57,30 +57,16 @@ impl_Bonobo_ControlFrame_activated (PortableServer_Servant  servant,
 			 control_frame_signals [ACTIVATED], state);
 }
 
-static void
-impl_Bonobo_ControlFrame_deactivate_and_undo (PortableServer_Servant  servant,
-					     CORBA_Environment       *ev)
-{
-	BonoboControlFrame *control_frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
-
-	gtk_signal_emit (GTK_OBJECT (control_frame),
-			 control_frame_signals [ACTIVATED], FALSE);
-
-	gtk_signal_emit (GTK_OBJECT (control_frame),
-			 control_frame_signals [UNDO_LAST_OPERATION]);
-}
-
-
 static Bonobo_UIContainer
 impl_Bonobo_ControlFrame_get_ui_handler (PortableServer_Servant  servant,
 					 CORBA_Environment      *ev)
 {
 	BonoboControlFrame *control_frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
 
-	if (control_frame->priv->uih == NULL)
+	if (control_frame->priv->ui_container == NULL)
 		return CORBA_OBJECT_NIL;
 
-	return bonobo_object_dup_ref (control_frame->priv->uih, ev);
+	return bonobo_object_dup_ref (control_frame->priv->ui_container, ev);
 }
 
 static Bonobo_PropertyBag
@@ -179,6 +165,7 @@ bonobo_control_frame_autoactivate_focus_out (GtkWidget     *widget,
 	return FALSE;
 }
 
+
 static void
 bonobo_control_frame_socket_state_changed (GtkWidget    *socket,
 					   GtkStateType  previous_state,
@@ -194,6 +181,16 @@ bonobo_control_frame_socket_state_changed (GtkWidget    *socket,
 		GTK_WIDGET_STATE (control_frame->priv->socket));
 }
 
+
+static void
+bonobo_control_frame_socket_destroy (GtkWidget          *socket,
+				     BonoboControlFrame *control_frame)
+{
+	gtk_widget_unref (control_frame->priv->socket);
+	control_frame->priv->socket = NULL;
+}
+
+
 static void
 bonobo_control_frame_set_remote_window (GtkWidget          *socket,
 					BonoboControlFrame *control_frame)
@@ -228,38 +225,9 @@ bonobo_control_frame_set_remote_window (GtkWidget          *socket,
 	CORBA_exception_free (&ev);
 }
 
-/**
- * bonobo_control_frame_construct:
- * @control_frame: The #BonoboControlFrame object to be initialized.
- * @corba_control_frame: A CORBA object for the Bonobo_ControlFrame interface.
- * @uih: A CORBA object for the UIContainer for the container application.
- *
- * Initializes @control_frame with the parameters.
- *
- * Returns: the initialized BonoboControlFrame object @control_frame that implements the
- * Bonobo::ControlFrame CORBA service.
- */
-BonoboControlFrame *
-bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
-				Bonobo_ControlFrame  corba_control_frame,
-				Bonobo_UIContainer   uih)
+static void
+bonobo_control_frame_create_socket (BonoboControlFrame  *control_frame)
 {
-	g_return_val_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame), NULL);
-
-	bonobo_object_construct (BONOBO_OBJECT (control_frame), corba_control_frame);
-
-	/*
-	 * See ui-faq.txt if this dies on you.
-	 */
-	if (uih != CORBA_OBJECT_NIL) {
-		CORBA_Environment ev;
-		CORBA_exception_init (&ev);
-		g_assert (CORBA_Object_is_a (uih, "IDL:Bonobo/UIContainer:1.0", &ev));
-		CORBA_exception_free (&ev);
-	}
-
-	control_frame->priv->uih = uih;
-
 	/*
 	 * Now create the GtkSocket which will be used to embed
 	 * the Control.
@@ -290,18 +258,25 @@ bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
 			    control_frame);
 
 	/*
-	 * Finally, create a box to hold the socket; this no-window
-	 * container is needed solely for the sake of bypassing
-	 * plug/socket in the local case.
+	 * Setup a handler for socket destroy.
 	 */
-	control_frame->priv->container = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (control_frame->priv->container), 0);
-	gtk_box_pack_end (GTK_BOX (control_frame->priv->container),
-			  control_frame->priv->socket,
-			  TRUE, TRUE, 0);
-	gtk_widget_ref (control_frame->priv->container);
-	gtk_object_sink (GTK_OBJECT (control_frame->priv->container));
-	gtk_widget_show (control_frame->priv->container);
+	gtk_signal_connect (GTK_OBJECT (control_frame->priv->socket),
+			    "destroy",
+			    bonobo_control_frame_socket_destroy,
+			    control_frame);
+
+	/*
+	 * Ref the socket so we can handle the case of the control destroying it for bypass.
+	 */
+	gtk_object_ref (GTK_OBJECT (control_frame->priv->socket));
+
+
+	/*
+	 * Pack into the hack box
+	 */
+	gtk_box_pack_start (GTK_BOX (control_frame->priv->container),
+			    control_frame->priv->socket,
+			    TRUE, TRUE, 0);
 
 	/*
 	 * When the socket is realized, we pass its Window ID to our
@@ -311,19 +286,66 @@ bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
 			    "realize",
 			    GTK_SIGNAL_FUNC (bonobo_control_frame_set_remote_window),
 			    control_frame);
+}
+				
+
+/**
+ * bonobo_control_frame_construct:
+ * @control_frame: The #BonoboControlFrame object to be initialized.
+ * @corba_control_frame: A CORBA object for the Bonobo_ControlFrame interface.
+ * @ui_container: A CORBA object for the UIContainer for the container application.
+ *
+ * Initializes @control_frame with the parameters.
+ *
+ * Returns: the initialized BonoboControlFrame object @control_frame that implements the
+ * Bonobo::ControlFrame CORBA service.
+ */
+BonoboControlFrame *
+bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
+				Bonobo_ControlFrame  corba_control_frame,
+				Bonobo_UIContainer   ui_container)
+{
+	g_return_val_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame), NULL);
+
+	bonobo_object_construct (BONOBO_OBJECT (control_frame), corba_control_frame);
+
+	/*
+	 * See ui-faq.txt if this dies on you.
+	 */
+	if (ui_container != CORBA_OBJECT_NIL) {
+		CORBA_Environment ev;
+		CORBA_exception_init (&ev);
+		g_assert (CORBA_Object_is_a (ui_container, "IDL:Bonobo/UIContainer:1.0", &ev));
+		CORBA_exception_free (&ev);
+	}
+
+	control_frame->priv->ui_container = ui_container;
+
+	/*
+	 * Finally, create a box to hold the socket; this no-window
+	 * container is needed solely for the sake of bypassing
+	 * plug/socket in the local case.
+	 */
+	control_frame->priv->container = gtk_hbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (control_frame->priv->container), 0);
+	gtk_widget_ref (control_frame->priv->container);
+	gtk_object_sink (GTK_OBJECT (control_frame->priv->container));
+	gtk_widget_show (control_frame->priv->container);
+
+	bonobo_control_frame_create_socket (control_frame);
 
 	return control_frame;
 }
 
 /**
  * bonobo_control_frame_new:
- * @uih: The #Bonobo_UIContainer for the container application.
+ * @ui_container: The #Bonobo_UIContainer for the container application.
  *
  * Returns: BonoboControlFrame object that implements the
  * Bonobo::ControlFrame CORBA service. 
  */
 BonoboControlFrame *
-bonobo_control_frame_new (Bonobo_UIContainer uih)
+bonobo_control_frame_new (Bonobo_UIContainer ui_container)
 {
 	Bonobo_ControlFrame corba_control_frame;
 	BonoboControlFrame *control_frame;
@@ -336,7 +358,7 @@ bonobo_control_frame_new (Bonobo_UIContainer uih)
 		return NULL;
 	}
 
-	return bonobo_control_frame_construct (control_frame, corba_control_frame, uih);
+	return bonobo_control_frame_construct (control_frame, corba_control_frame, ui_container);
 }
 
 static void
@@ -349,6 +371,13 @@ bonobo_control_frame_destroy (GtkObject *object)
 
 	gtk_widget_unref (control_frame->priv->container);
 
+	if (control_frame->priv->socket) {
+		gtk_signal_disconnect_by_data (GTK_OBJECT (control_frame->priv->socket),
+					       control_frame);
+		gtk_widget_unref (control_frame->priv->socket);
+		control_frame->priv->socket = NULL;
+	}
+
 	g_free (control_frame->priv);
 	
 	GTK_OBJECT_CLASS (bonobo_control_frame_parent_class)->destroy (object);
@@ -365,7 +394,6 @@ bonobo_control_frame_get_epv (void)
 	epv = g_new0 (POA_Bonobo_ControlFrame__epv, 1);
 
 	epv->activated              = impl_Bonobo_ControlFrame_activated;
-	epv->deactivate_and_undo    = impl_Bonobo_ControlFrame_deactivate_and_undo;
 	epv->get_ui_handler         = impl_Bonobo_ControlFrame_get_ui_handler;
 	epv->queue_resize           = impl_Bonobo_ControlFrame_queue_resize;
 	epv->activate_uri           = impl_Bonobo_ControlFrame_activate_uri;
@@ -413,14 +441,6 @@ bonobo_control_frame_class_init (BonoboControlFrameClass *klass)
 				GTK_TYPE_BOOL);
 
 	
-	control_frame_signals [UNDO_LAST_OPERATION] =
-		gtk_signal_new ("undo_last_operation",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (BonoboControlFrameClass, undo_last_operation),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-
 	control_frame_signals [ACTIVATE_URI] =
 		gtk_signal_new ("activate_uri",
 				GTK_RUN_LAST,
@@ -681,19 +701,19 @@ bonobo_control_frame_get_autostate (BonoboControlFrame *control_frame)
 
 
 /**
- * bonobo_control_frame_get_ui_handler:
+ * bonobo_control_frame_get_ui_container:
  * @control_frame: A BonoboControlFrame object.
 
  * Returns: The Bonobo_UIContainer object reference associated with this
- * ControlFrame.  This uih is specified when the ControlFrame is
+ * ControlFrame.  This ui_container is specified when the ControlFrame is
  * created.  See bonobo_control_frame_new().
  */
 Bonobo_UIContainer
-bonobo_control_frame_get_ui_handler (BonoboControlFrame *control_frame)
+bonobo_control_frame_get_ui_container (BonoboControlFrame *control_frame)
 {
 	g_return_val_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame), CORBA_OBJECT_NIL);
 
-	return control_frame->priv->uih;
+	return control_frame->priv->ui_container;
 }
 
 /**
@@ -731,10 +751,17 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 		bonobo_object_check_env (BONOBO_OBJECT (control_frame), control, &ev);
 	CORBA_exception_free (&ev);
 
+	/* 
+	 * Re-create the socket if it got destroyed by the Control before.
+	 */
+	if (control_frame->priv->socket == NULL) 
+		bonobo_control_frame_create_socket (control_frame);
+
 	/*
 	 * If the socket is realized, then we transfer the
 	 * window ID to the remote control.
 	 */
+
 	if (GTK_WIDGET_REALIZED (control_frame->priv->socket))
 		bonobo_control_frame_set_remote_window (control_frame->priv->socket,
 							control_frame);
