@@ -58,10 +58,10 @@ typedef struct {
 			   const char        *xml,
 			   CORBA_Environment *ev);
 
-	char *(*xml_get)  (BonoboUIComponent *component,
-			   const char        *path,
-			   gboolean           recurse,
-			   CORBA_Environment *ev);
+	CORBA_char *(*xml_get) (BonoboUIComponent *component,
+				const char        *path,
+				gboolean           recurse,
+				CORBA_Environment *ev);
 
 	void (*xml_rm)    (BonoboUIComponent *component,
 			   const char        *path,
@@ -131,6 +131,11 @@ void               bonobo_ui_component_set          (BonoboUIComponent  *compone
 						     const char         *xml,
 						     CORBA_Environment  *ev);
 
+void               bonobo_ui_component_set_translate(BonoboUIComponent  *component,
+						     const char         *path,
+						     const char         *xml,
+						     CORBA_Environment  *ev);
+
 void               bonobo_ui_component_set_tree     (BonoboUIComponent  *component,
 						     const char         *path,
 						     BonoboUINode       *node,
@@ -144,7 +149,7 @@ gboolean           bonobo_ui_component_path_exists  (BonoboUIComponent  *compone
 						     const char         *path,
 						     CORBA_Environment  *ev);
 
-char              *bonobo_ui_component_get          (BonoboUIComponent  *component,
+CORBA_char        *bonobo_ui_component_get          (BonoboUIComponent  *component,
 						     const char         *path,
 						     gboolean            recurse,
 						     CORBA_Environment  *ev);
