@@ -24,12 +24,10 @@
 #ifndef _BONOBO_UI_TOOLBAR_ITEM_H_
 #define _BONOBO_UI_TOOLBAR_ITEM_H_
 
+#include <libgnome/gnome-defs.h>
 #include <gtk/gtkbin.h>
 
-#ifdef __cplusplus
-extern "C" {
-#pragma }
-#endif /* __cplusplus */
+BEGIN_GNOME_DECLS
 
 #define BONOBO_TYPE_UI_TOOLBAR_ITEM			(bonobo_ui_toolbar_item_get_type ())
 #define BONOBO_UI_TOOLBAR_ITEM(obj)			(GTK_CHECK_CAST ((obj), BONOBO_TYPE_UI_TOOLBAR_ITEM, BonoboUIToolbarItem))
@@ -81,8 +79,19 @@ struct _BonoboUIToolbarItemClass {
 };
 
 
-GtkType                  bonobo_ui_toolbar_item_get_type         (void);
-GtkWidget               *bonobo_ui_toolbar_item_new              (void);
+GtkType                   bonobo_ui_toolbar_item_get_type         (void);
+GtkWidget                *bonobo_ui_toolbar_item_new              (void);
+void                      bonobo_ui_toolbar_item_set_tooltip      (BonoboUIToolbarItem      *item,
+								   GtkTooltips              *tooltips,
+								   const char               *tooltip);
+void                      bonobo_ui_toolbar_item_set_state        (BonoboUIToolbarItem      *item,
+								   const char               *new_state);
+void                      bonobo_ui_toolbar_item_set_orientation  (BonoboUIToolbarItem      *item,
+								   GtkOrientation            orientation);
+GtkOrientation            bonobo_ui_toolbar_item_get_orientation  (BonoboUIToolbarItem      *item);
+void                      bonobo_ui_toolbar_item_set_style        (BonoboUIToolbarItem      *item,
+								   BonoboUIToolbarItemStyle  style);
+BonoboUIToolbarItemStyle  bonobo_ui_toolbar_item_get_style        (BonoboUIToolbarItem      *item);
 
 void			 bonobo_ui_toolbar_item_set_minimum_width(BonoboUIToolbarItem *item,
 								  int minimum_width);
@@ -102,10 +111,19 @@ void                     bonobo_ui_toolbar_item_set_style        (BonoboUIToolba
 BonoboUIToolbarItemStyle bonobo_ui_toolbar_item_get_style        (BonoboUIToolbarItem     *item);
 
 /* FIXME ugly names.  */
-void                     bonobo_ui_toolbar_item_set_want_label   (BonoboUIToolbarItem     *button_item,
-								  gboolean                 prefer_text);
-gboolean                 bonobo_ui_toolbar_item_get_want_label   (BonoboUIToolbarItem     *button_item);
-void                     bonobo_ui_toolbar_item_activate         (BonoboUIToolbarItem     *item);
+void                      bonobo_ui_toolbar_item_set_want_label   (BonoboUIToolbarItem      *button_item,
+								   gboolean                  prefer_text);
+gboolean                  bonobo_ui_toolbar_item_get_want_label   (BonoboUIToolbarItem      *button_item);
+
+void                      bonobo_ui_toolbar_item_set_expandable   (BonoboUIToolbarItem      *button_item,
+								   gboolean                  expandable);
+gboolean                  bonobo_ui_toolbar_item_get_expandable   (BonoboUIToolbarItem      *button_item);
+
+void                      bonobo_ui_toolbar_item_set_pack_end     (BonoboUIToolbarItem      *button_item,
+								   gboolean                  expandable);
+gboolean                  bonobo_ui_toolbar_item_get_pack_end     (BonoboUIToolbarItem      *button_item);
+
+void                      bonobo_ui_toolbar_item_activate         (BonoboUIToolbarItem     *item);
 
 #ifdef __cplusplus
 }
