@@ -37,10 +37,13 @@
 static GtkContainerClass *parent_class = NULL;
 
 enum {
-  ARG_0,
-  ARG_ORIENTATION,
-  ARG_IS_FLOATING,
+	ARG_0,
+	ARG_ORIENTATION,
+	ARG_IS_FLOATING,
+	ARG_PREFERRED_WIDTH,
+	ARG_PREFERRED_HEIGHT
 };
+
 struct _BonoboUIToolbarPrivate {
 	/* The orientation of this toolbar.  */
 	GtkOrientation orientation;
@@ -56,6 +59,9 @@ struct _BonoboUIToolbarPrivate {
 	int max_width, max_height;
 	int total_width, total_height;
 
+	/* position of left edge of left-most pack-end item */
+	int end_position;
+
 	/* List of all the items in the toolbar.  Both the ones that have been
            unparented because they don't fit, and the ones that are visible.
            The BonoboUIToolbarPopupItem is not here though.  */
@@ -66,7 +72,7 @@ struct _BonoboUIToolbarPrivate {
 	GList *first_not_fitting_item;
 
 	/* The pop-up button.  When clicked, it pops up a window with all the
-X           items that don't fit.  */
+           items that don't fit.  */
 	BonoboUIToolbarItem *popup_item;
 
 	/* The window we pop-up when the pop-up item is clicked.  */
@@ -214,7 +220,9 @@ create_popup_window (BonoboUIToolbar *toolbar)
 
 		item_widget = GTK_WIDGET (p->data);
 
-		if (! GTK_WIDGET_VISIBLE (item_widget))
+		if (! GTK_WIDGET_VISIBLE (item_widget) ||
+			bonobo_ui_toolbar_item_get_pack_end (BONOBO_UI_TOOLBAR_ITEM (item_widget)))
+
 			continue;
 
 		if (item_widget->parent != NULL)
@@ -437,13 +445,13 @@ allocate_popup_item (BonoboUIToolbar *toolbar)
 	popup_item_allocation.y = toolbar_allocation->y;
 
 	if (priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
-		popup_item_allocation.x      += toolbar_allocation->width - popup_item_requisition.width - border_width;
+		popup_item_allocation.x      = priv->end_position - popup_item_requisition.width - border_width;
 		popup_item_allocation.y      += border_width;
 		popup_item_allocation.width  = popup_item_requisition.width;
 		popup_item_allocation.height = toolbar_allocation->height - 2 * border_width;
 	} else {
 		popup_item_allocation.x      += border_width;
-		popup_item_allocation.y      += toolbar_allocation->height - popup_item_requisition.height - border_width;
+		popup_item_allocation.y      = priv->end_position - popup_item_requisition.height - border_width;
 		popup_item_allocation.width  = toolbar_allocation->width - 2 * border_width;
 		popup_item_allocation.height = popup_item_requisition.height;
 	}
@@ -481,7 +489,7 @@ setup_popup_item (BonoboUIToolbar *toolbar)
 }
 
 /*
- *  This is a dirty hack.  We cannot hide the items with gtk_widget_hide ()
+ * This is a dirty hack.  We cannot hide the items with gtk_widget_hide ()
  * because we want to let the user be in control of the physical hidden/shown
  * state, so we just move the widget to a non-visible area.
  */
@@ -502,130 +510,183 @@ hide_not_fitting_items (BonoboUIToolbar *toolbar)
 	child_allocation.width  = 1;
 	child_allocation.height = 1;
 
-	for (p = priv->first_not_fitting_item; p != NULL; p = p->next)
+	for (p = priv->first_not_fitting_item; p != NULL; p = p->next) {
+		if (bonobo_ui_toolbar_item_get_pack_end (BONOBO_UI_TOOLBAR_ITEM (p->data)))
+			continue;
 		gtk_widget_size_allocate (GTK_WIDGET (p->data), &child_allocation);
+	}
 }
 
 static void
-size_allocate_horizontally (BonoboUIToolbar *toolbar,
-			    const GtkAllocation *allocation)
+size_allocate_helper (BonoboUIToolbar *toolbar,
+		      const GtkAllocation *allocation)
 {
 	BonoboUIToolbarPrivate *priv;
 	GtkAllocation child_allocation;
-	GList *p;
-	int available_width;
+	BonoboUIToolbarItem *item;
+	GtkRequisition child_requisition;
 	int border_width;
+	int space_required;
+	int available_space;
+	int extra_space;
+	int num_expandable_items;
+	int popup_item_size;
+	gboolean first_expandable;
+	GList *p;
 
 	GTK_WIDGET (toolbar)->allocation = *allocation;
 
 	priv = toolbar->priv;
 
 	border_width = GTK_CONTAINER (toolbar)->border_width;
+	popup_item_size = get_popup_item_size (toolbar);
 
-	available_width = allocation->width - 2 * border_width;
-	if (available_width < 0)
-		available_width = 0;
+	if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+		available_space = MAX ((int) allocation->width - 2 * border_width, popup_item_size);
+	else
+		available_space = MAX ((int) allocation->height - 2 * border_width, popup_item_size);
 
 	child_allocation.x = allocation->x + border_width;
 	child_allocation.y = allocation->y + border_width;
 
-	priv->first_not_fitting_item = NULL;
-
-	for (p = priv->items; p != NULL; p = p->next) {
-		GtkWidget *widget;
-		GtkRequisition child_requisition;
-
-		widget = GTK_WIDGET (p->data);
-		if (! GTK_WIDGET_VISIBLE (widget) || widget->parent != GTK_WIDGET (toolbar))
-			continue;
-
-		gtk_widget_get_child_requisition (widget, &child_requisition);
-
-		if (available_width < child_requisition.width) {
-			priv->first_not_fitting_item = p;
-			break;
-		}
-
-		child_allocation.width  = child_requisition.width;
-		child_allocation.height = priv->max_height;
-
-		gtk_widget_size_allocate (widget, &child_allocation);
-
-		child_allocation.x += child_allocation.width;
-		available_width    -= child_allocation.width;
-	}
-
-	/* Something did not fit allocate the arrow */
-	if (p != NULL) {
-		/* Does the arrow fit ? */
-		available_width -= get_popup_item_size (toolbar);
-
-		/* Damn, it does not.  Remove the previous element if possible */
-		if (available_width < 0 && p->prev != NULL)
-			priv->first_not_fitting_item = p->prev;
-	}
-
-	hide_not_fitting_items (toolbar);
-	setup_popup_item (toolbar);
-}
-
-static void
-size_allocate_vertically (BonoboUIToolbar *toolbar,
-			  const GtkAllocation *allocation)
-{
-	BonoboUIToolbarPrivate *priv;
-	GtkAllocation child_allocation;
-	GList *p;
-	int available_height;
-	int border_width;
-
-	GTK_WIDGET (toolbar)->allocation = *allocation;
-
-	priv = toolbar->priv;
-
-	border_width = GTK_CONTAINER (toolbar)->border_width;
-
-	available_height = allocation->height - 2 * border_width;
-	if (available_height < 0)
-		available_height = 0;
-
-	child_allocation.x = allocation->x + border_width;
-	child_allocation.y = allocation->y + border_width;
-
-	priv->first_not_fitting_item = NULL;
-
-	for (p = priv->items; p != NULL; p = p->next) {
-		GtkWidget *widget;
-		GtkRequisition child_requisition;
-
-		widget = GTK_WIDGET (p->data);
-		if (! GTK_WIDGET_VISIBLE (widget) || widget->parent != GTK_WIDGET (toolbar))
-			continue;
-
-		gtk_widget_get_child_requisition (widget, &child_requisition);
-
-		if (available_height < child_requisition.height) {
-			priv->first_not_fitting_item = p;
-			break;
-		}
-
-		child_allocation.width  = priv->max_width;
+	/* 
+	 * if there is exactly one toolbar item, handle it specially, by giving it all of the available space,
+	 * even if it doesn't fit, since we never want everything in the pop-up.
+	 */	 
+	if (priv->items != NULL && priv->items->next == NULL) {
+		item = BONOBO_UI_TOOLBAR_ITEM (priv->items->data);		
+		gtk_widget_get_child_requisition (GTK_WIDGET (item), &child_requisition);
+		child_allocation.width = child_requisition.width;
 		child_allocation.height = child_requisition.height;
 
-		gtk_widget_size_allocate (widget, &child_allocation);
-
-		child_allocation.y += child_allocation.height;
-		available_height   -= child_allocation.height;
+		if (bonobo_ui_toolbar_item_get_expandable (item)) {
+			if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+				child_allocation.width = available_space;
+			else
+				child_allocation.height = available_space;
+		
+		} 
+		gtk_widget_size_allocate (GTK_WIDGET (item), &child_allocation);		
+		
+		return;
 	}
 
-	/* Something did not fit allocate the arrow */
-	if (p != NULL) {
-		/* Does the arrow fit ? */
-		available_height -= get_popup_item_size (toolbar);
+	/* first, make a pass through the items to layout the ones that are packed on the right */
+	priv->end_position = allocation->x + available_space;
+	for (p = priv->items; p != NULL; p = p->next) {
 
-		/* Damn, it does not.  Remove the previous element if possible */
-		if (available_height < 0 && p->prev != NULL)
-			priv->first_not_fitting_item = p->prev;
+		item = BONOBO_UI_TOOLBAR_ITEM (p->data);
+		if (! bonobo_ui_toolbar_item_get_pack_end (item))
+			continue;
+
+		gtk_widget_get_child_requisition (GTK_WIDGET (item), &child_requisition);
+
+		if (priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
+			available_space -= child_requisition.width;
+			priv->end_position -= child_requisition.width;
+			
+			child_allocation.x = priv->end_position;
+			child_allocation.width = child_requisition.width;
+			child_allocation.height = priv->max_height;
+		} else {
+			available_space -= child_requisition.height;
+			priv->end_position -= child_requisition.height;
+			
+			child_allocation.y = priv->end_position;
+			child_allocation.height = child_requisition.height;
+			child_allocation.width = priv->max_width;
+		}
+		
+		gtk_widget_size_allocate (GTK_WIDGET (item), &child_allocation);
+	}
+	
+	/* make a pass through the items to determine how many fit */	
+	space_required = 0;
+	num_expandable_items = 0;
+
+	child_allocation.x = allocation->x + border_width;
+	child_allocation.y = allocation->y + border_width;
+
+	for (p = priv->items; p != NULL; p = p->next) {
+		int item_size;
+
+		item = BONOBO_UI_TOOLBAR_ITEM (p->data);
+		if (! GTK_WIDGET_VISIBLE (item) || GTK_WIDGET (item)->parent != GTK_WIDGET (toolbar) ||
+			bonobo_ui_toolbar_item_get_pack_end (item))
+			continue;
+
+		gtk_widget_get_child_requisition (GTK_WIDGET (item), &child_requisition);
+
+		if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+			item_size = child_requisition.width;
+		else
+			item_size = child_requisition.height;
+
+		if (p->next == NULL) {
+			if (space_required + item_size > available_space)
+				break;
+		} else {
+			if (space_required + item_size > available_space - popup_item_size)
+				break;
+		}
+
+		space_required += item_size;
+
+		if (bonobo_ui_toolbar_item_get_expandable (item))
+			num_expandable_items ++;
+	}
+
+	priv->first_not_fitting_item = p;
+
+	/* determine the amount of space available for expansion */
+	if (priv->first_not_fitting_item != NULL) {
+		extra_space = 0;
+	} else {
+		extra_space = available_space - space_required;
+		if (priv->first_not_fitting_item != NULL)
+			extra_space -= popup_item_size;
+	}
+
+	first_expandable = FALSE;
+
+	for (p = priv->items; p != priv->first_not_fitting_item; p = p->next) {
+		BonoboUIToolbarItem *item;
+		GtkRequisition child_requisition;
+		int expansion_amount;
+
+		item = BONOBO_UI_TOOLBAR_ITEM (p->data);
+		if (! GTK_WIDGET_VISIBLE (item) || GTK_WIDGET (item)->parent != GTK_WIDGET (toolbar) ||
+			bonobo_ui_toolbar_item_get_pack_end (item))
+			continue;
+
+		gtk_widget_get_child_requisition (GTK_WIDGET (item), &child_requisition);
+
+		if (! bonobo_ui_toolbar_item_get_expandable (item)) {
+			expansion_amount = 0;
+		} else {
+			g_assert (num_expandable_items != 0);
+
+			expansion_amount = extra_space / num_expandable_items;
+			if (first_expandable) {
+				expansion_amount += extra_space % num_expandable_items;
+				first_expandable = FALSE;
+			}
+		}
+
+		if (priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
+			child_allocation.width  = child_requisition.width + expansion_amount;
+			child_allocation.height = priv->max_height;
+		} else {
+			child_allocation.width  = priv->max_width;
+			child_allocation.height = child_requisition.height + expansion_amount;
+		}
+
+		gtk_widget_size_allocate (GTK_WIDGET (item), &child_allocation);
+
+		if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+			child_allocation.x += child_allocation.width;
+		else
+			child_allocation.y += child_allocation.height;
 	}
 
 	hide_not_fitting_items (toolbar);
@@ -662,6 +723,8 @@ impl_destroy (GtkObject *object)
 	gtk_object_unref (GTK_OBJECT (priv->tooltips));
 	priv->tooltips = NULL;
 
+	g_free (priv);
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy != NULL)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -700,11 +763,11 @@ impl_size_request (GtkWidget *widget,
 		gtk_widget_size_request (GTK_WIDGET (priv->popup_item), &popup_item_requisition);
 
 		if (priv->orientation == GTK_ORIENTATION_HORIZONTAL) {
-			requisition->width  = MAX (popup_item_requisition.width,  priv->total_width);
+			requisition->width  = popup_item_requisition.width;
 			requisition->height = MAX (popup_item_requisition.height, priv->max_height);
 		} else {
 			requisition->width  = MAX (popup_item_requisition.width,  priv->max_width);
-			requisition->height = MAX (popup_item_requisition.height, priv->total_height);
+			requisition->height = popup_item_requisition.height;
 		}
 	}
 
@@ -722,10 +785,7 @@ impl_size_allocate (GtkWidget *widget,
 	toolbar = BONOBO_UI_TOOLBAR (widget);
 	priv = toolbar->priv;
 
-	if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
-		size_allocate_horizontally (toolbar, allocation);
-	else
-		size_allocate_vertically (toolbar, allocation);
+	size_allocate_helper (toolbar, allocation);
 }
 
 static void
@@ -860,11 +920,11 @@ impl_remove (GtkContainer *container,
 	toolbar = BONOBO_UI_TOOLBAR (container);
 	priv = toolbar->priv;
 
-	gtk_widget_unparent (child);
-
 	if (child == GTK_WIDGET (priv->popup_item))
 		priv->popup_item = NULL;
 
+	gtk_widget_unparent (child);
+
 	gtk_widget_queue_resize (GTK_WIDGET (container));
 }
 
@@ -955,44 +1015,55 @@ impl_set_style (BonoboUIToolbar *toolbar,
 }
 
 static void
-impl_get_arg(GtkObject* obj, GtkArg* arg, guint arg_id)
+impl_get_arg (GtkObject *obj,
+	      GtkArg *arg,
+	      guint arg_id)
 {
-  BonoboUIToolbar *toolbar = BONOBO_UI_TOOLBAR (obj);
-  BonoboUIToolbarPrivate *priv = toolbar->priv;
+	BonoboUIToolbar *toolbar = BONOBO_UI_TOOLBAR (obj);
+	BonoboUIToolbarPrivate *priv = toolbar->priv;
 
-  switch (arg_id) {
-  case ARG_ORIENTATION:
-    GTK_VALUE_UINT(*arg) = bonobo_ui_toolbar_get_orientation (toolbar);
-    break;
-
-  case ARG_IS_FLOATING :
-    GTK_VALUE_BOOL(*arg) = priv->is_floating;
-    break;
-
-  default:
-    break;
-  };
+	switch (arg_id) {
+	case ARG_ORIENTATION:
+		GTK_VALUE_UINT (*arg) = bonobo_ui_toolbar_get_orientation (toolbar);
+		break;
+	case ARG_IS_FLOATING:
+		GTK_VALUE_BOOL (*arg) = priv->is_floating;
+		break;
+	case ARG_PREFERRED_WIDTH:
+		if (bonobo_ui_toolbar_get_orientation (toolbar) == GTK_ORIENTATION_HORIZONTAL)
+			GTK_VALUE_UINT (*arg) = priv->total_width;
+		else
+			GTK_VALUE_UINT (*arg) = priv->max_width;
+		break;
+	case ARG_PREFERRED_HEIGHT:
+		if (bonobo_ui_toolbar_get_orientation (toolbar) == GTK_ORIENTATION_HORIZONTAL)
+			GTK_VALUE_UINT (*arg) = priv->max_height;
+		else
+			GTK_VALUE_UINT (*arg) = priv->total_height;
+		break;
+	default:
+		break;
+	};
 }
 
 static void
-impl_set_arg(GtkObject* obj, GtkArg* arg, guint arg_id)
+impl_set_arg (GtkObject *obj,
+	      GtkArg *arg,
+	      guint arg_id)
 {
-  BonoboUIToolbar *toolbar = BONOBO_UI_TOOLBAR (obj);
-  BonoboUIToolbarPrivate *priv = toolbar->priv;
+	BonoboUIToolbar *toolbar = BONOBO_UI_TOOLBAR (obj);
+	BonoboUIToolbarPrivate *priv = toolbar->priv;
 
-  switch (arg_id) {
-
-  case ARG_ORIENTATION:
-    bonobo_ui_toolbar_set_orientation (toolbar, GTK_VALUE_UINT(*arg));
-    break;
-
-  case ARG_IS_FLOATING :
-    priv->is_floating = GTK_VALUE_BOOL(*arg);
-    break;
-
-  default:
-    break;
-  };
+	switch (arg_id) {
+	case ARG_ORIENTATION:
+		bonobo_ui_toolbar_set_orientation (toolbar, GTK_VALUE_UINT(*arg));
+		break;
+	case ARG_IS_FLOATING:
+		priv->is_floating = GTK_VALUE_BOOL(*arg);
+		break;
+	default:
+		break;
+	};
 }
 
 static void
@@ -1029,6 +1100,11 @@ class_init (BonoboUIToolbarClass *toolbar_class)
 	gtk_object_add_arg_type("BonoboUIToolbar::is_floating",
 				GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_IS_FLOATING);
 
+	gtk_object_add_arg_type("BonoboUIToolbar::preferred_width",
+				GTK_TYPE_UINT, GTK_ARG_READABLE, ARG_PREFERRED_WIDTH);
+	gtk_object_add_arg_type("BonoboUIToolbar::preferred_height",
+				GTK_TYPE_UINT, GTK_ARG_READABLE, ARG_PREFERRED_HEIGHT);
+
 	signals[SET_ORIENTATION]
 		= gtk_signal_new ("set_orientation",
 				  GTK_RUN_LAST,
@@ -1246,25 +1322,10 @@ bonobo_ui_toolbar_insert (BonoboUIToolbar *toolbar,
 	gtk_widget_queue_resize (GTK_WIDGET (toolbar));
 }
 
-static void
-containee_destroy_cb (GtkWidget *widget,
-		      BonoboUIToolbarPrivate *priv)
+GList *
+bonobo_ui_toolbar_get_children (BonoboUIToolbar *toolbar)
 {
-	if (widget != (GtkWidget *)priv->popup_item)
-		gtk_widget_destroy (widget);
-	else
-		g_warning ("Skipping '%p' : popup item", widget);
-}
+	g_return_val_if_fail (BONOBO_IS_UI_TOOLBAR (toolbar), NULL);
 
-void
-bonobo_ui_toolbar_clean (BonoboUIToolbar *toolbar)
-{
-	g_return_if_fail (toolbar != NULL);
-	g_return_if_fail (BONOBO_IS_UI_TOOLBAR (toolbar));
-
-	g_warning ("UI toolbar clean");
-
-	gtk_container_foreach (GTK_CONTAINER (toolbar),
-			       (GtkCallback) containee_destroy_cb,
-			       toolbar->priv);
+	return g_list_copy (toolbar->priv->items);
 }
