@@ -93,4 +93,8 @@ void            bonobo_ui_util_accel_parse  (char              *name,
 					     guint             *accelerator_key,
 					     GdkModifierType   *accelerator_mods);
 
+char           *bonobo_ui_util_decode_str   (const char *str, gboolean *err);
+
+char           *bonobo_ui_util_encode_str   (const char *str);
+
 #endif /* _BONOBO_UI_XML_UTIL_H_ */
