@@ -30,6 +30,7 @@
 
 #undef STATE_SYNC_DEBUG
 #undef WIDGET_SYNC_DEBUG
+#undef XML_MERGE_DEBUG
 
 #define	BINDING_MOD_MASK()				\
 	(gtk_accelerator_get_default_mod_mask () | GDK_RELEASE_MASK)
@@ -84,15 +85,6 @@ typedef struct {
 	Bonobo_Unknown  object;
 } NodeInfo;
 
-static void
-info_dump_fn (BonoboUIXmlData *a)
-{
-	NodeInfo *info = (NodeInfo *) a;
-
-	fprintf (stderr, " '%s' widget %8p object %8p type %d\n",
-		 (char *)a->id, info->widget, info->object, info->type);
-}
-
 static BonoboUIXmlData *
 info_new_fn (void)
 {
@@ -128,6 +120,35 @@ widget_get_node (GtkWidget *widget)
 				    WIDGET_NODE_KEY);
 }
 
+static void
+info_dump_fn (BonoboUIXml *tree, BonoboUINode *node)
+{
+	NodeInfo *info = bonobo_ui_xml_get_data (tree, node);
+
+	if (info) {
+		fprintf (stderr, " '%15s' object %8p type %d ",
+			 (char *)info->parent.id, info->object, info->type);
+
+		if (info->widget) {
+			BonoboUINode *attached_node = widget_get_node (info->widget);
+
+			fprintf (stderr, "widget '%8p' with node '%8p' attached ",
+				 info->widget, attached_node);
+
+			if (attached_node == NULL)
+				fprintf (stderr, "is NULL\n");
+
+			else if (attached_node != node)
+				fprintf (stderr, "Serious mismatch attaches should be '%8p'\n",
+					 node);
+			else
+				fprintf (stderr, "and matching\n");
+ 		} else
+			fprintf (stderr, " no associated widget\n");
+	} else
+		fprintf (stderr, " very wierd no data on node '%p'\n", node);
+}
+
 /* We need to map the shell to the item */
 static GtkWidget *
 get_item_widget (GtkWidget *widget)
@@ -151,12 +172,14 @@ widget_set_node (GtkWidget    *widget,
 		 BonoboUINode *node)
 {
 	if (widget) {
+		GtkWidget *aux_widget;
+
 		gtk_object_set_data (GTK_OBJECT (widget),
 				     WIDGET_NODE_KEY, node);
 
-		if ((widget = get_item_widget (widget)))
+		if ((aux_widget = get_item_widget (widget)) != widget)
 
-			gtk_object_set_data (GTK_OBJECT (widget),
+			gtk_object_set_data (GTK_OBJECT (aux_widget),
 					     WIDGET_NODE_KEY, node);
 	}
 }
@@ -438,7 +461,6 @@ placeholder_sync (BonoboWinPrivate *priv,
 		bonobo_ui_node_free_string (txt);
 	}
 
-	/* FIXME: should we customize this ? */
 	if (!bonobo_ui_node_children (node))
 		show = FALSE;
 
@@ -507,6 +529,7 @@ sync_generic_widgets (BonoboWinPrivate *priv,
 				printf ("-- just syncing state --\n");
 #endif
 				ss (priv, a, b->data, parent);
+				(*pos)++;
 			} else {
 				NodeInfo   *info;
 				GtkWidget  *widget;
@@ -531,7 +554,7 @@ sync_generic_widgets (BonoboWinPrivate *priv,
 				}
 #ifdef WIDGET_SYNC_DEBUG
 				else
-					printf ("Failed to build widget");
+					printf ("Failed to build widget\n");
 #endif
 
 				nextb = b; /* NB. don't advance 'b' */
@@ -552,14 +575,19 @@ sync_generic_widgets (BonoboWinPrivate *priv,
 					 * on this 'b' node until a more favorable 'a'
 					 */
 					nextb = b;
-				} else
-					g_warning ("non dirty node, but widget mismatch "
-						   "a: '%s:%s', b: '%s:%s' '%p'",
-						   bonobo_ui_node_get_name (a),
-						   bonobo_ui_node_get_attr (a, "name"),
-						   bn ? bonobo_ui_node_get_name (bn) : "NULL",
-						   bn ? bonobo_ui_node_get_attr (bn, "name") : "NULL",
-						   info->widget);
+					g_assert (info->type | CUSTOM_WIDGET);
+#ifdef WIDGET_SYNC_DEBUG
+					printf ("not dirty & not same, but has no widget\n");
+#endif
+				} else {
+					printf ("Bonobo-WARNING **: non dirty node, but widget mismatch "
+						"a: '%s:%s', b: '%s:%s' '%p'\n\n",
+						bonobo_ui_node_get_name (a),
+						bonobo_ui_node_get_attr (a, "name"),
+						bn ? bonobo_ui_node_get_name (bn) : "NULL",
+						bn ? bonobo_ui_node_get_attr (bn, "name") : "NULL",
+						info->widget);
+				}
 			}
 #ifdef WIDGET_SYNC_DEBUG
 			else
@@ -568,13 +596,10 @@ sync_generic_widgets (BonoboWinPrivate *priv,
 			(*pos)++;
 		}
 
-		if (bonobo_ui_node_has_name (a, "placeholder")) {
-			(*pos)++;
+		if (bonobo_ui_node_has_name (a, "placeholder"))
 			sync_generic_widgets (priv, bonobo_ui_node_children (a),
 					      parent, &nextb, pos, sync_state,
 					      build_widget, make_placeholder);
-			(*pos)--;
-		}
 
 		a = bonobo_ui_node_next (a);
 	}
@@ -585,6 +610,31 @@ sync_generic_widgets (BonoboWinPrivate *priv,
 	*widgets = b;
 }
 
+static void
+check_excess_widgets (BonoboWinPrivate *priv, GList *wptr)
+{
+	if (wptr) {
+		GList *b;
+		int warned = 0;
+
+		for (b = wptr; b; b = b->next) {
+			BonoboUINode *node;
+
+			if (widget_is_special (b->data))
+				continue;
+			
+			if (!warned++)
+				printf ("Bonono-Warning **: Excess widgets at the "
+					"end of the container; wierd\n\n");
+
+			node = widget_get_node (b->data);
+			printf ("Widget type '%s' with node: '%s'\n",
+				gtk_type_name (GTK_OBJECT (b->data)->klass->type),
+				node ? bonobo_ui_xml_make_path (node) : "NULL");
+		}
+	}
+}
+
 static void
 cmd_set_dirty (BonoboWinPrivate *priv, BonoboUINode *cmd_node)
 {
@@ -597,24 +647,35 @@ cmd_set_dirty (BonoboWinPrivate *priv, BonoboUINode *cmd_node)
 	data->dirty = TRUE;
 }
 
-static void
+/*
+ * Returns: TRUE if hiddenness changed.
+ */
+static gboolean
 do_show_hide (GtkWidget *widget, BonoboUINode *node)
 {
-	char *txt;
+	char    *txt;
+	gboolean changed;
 
 	widget = get_item_widget (widget);
 
 	if (!widget)
-		return;
+		return FALSE;
 
 	if ((txt = bonobo_ui_node_get_attr (node, "hidden"))) {
 		if (atoi (txt)) {
+			changed = GTK_WIDGET_VISIBLE (widget);
 			gtk_widget_hide (widget);
-		} else
+		} else {
+			changed = !GTK_WIDGET_VISIBLE (widget);
 			gtk_widget_show (widget);
+		}
 		bonobo_ui_node_free_string (txt);
-	} else
+	} else {
+		changed = !GTK_WIDGET_VISIBLE (widget);
 		gtk_widget_show (widget);
+	}
+
+	return changed;
 }
 
 static void
@@ -728,21 +789,22 @@ cmd_get_node (BonoboWinPrivate *priv,
 	ret  = bonobo_ui_xml_get_path (priv->tree, path);
 
 	if (!ret) {
-		BonoboUIXmlData *data_from, *data_to;
+		BonoboUIXmlData *data_from;
+		BonoboUINode *commands;
+		BonoboUINode *node;
+
+		commands = bonobo_ui_node_new ("commands");
+		node     = bonobo_ui_node_new_child (commands, "cmd");
 
-		BonoboUINode *node = bonobo_ui_node_new ("cmd");
 		bonobo_ui_node_set_attr (node, "name", cmd_name);
 
+		data_from   = bonobo_ui_xml_get_data (priv->tree, from_node);
+
 		bonobo_ui_xml_merge (
-			priv->tree, "/commands", node, NULL);
+			priv->tree, "/", commands, data_from->id);
 		
 		ret = bonobo_ui_xml_get_path (priv->tree, path);
 		g_assert (ret != NULL);
-
-		data_from = bonobo_ui_xml_get_data (priv->tree, from_node);
-		data_to   = bonobo_ui_xml_get_data (priv->tree, ret);
-
-		data_to->id = data_from->id;
 	}
 
 	g_free (path);
@@ -934,8 +996,9 @@ replace_override_fn (GtkObject        *object,
 		     BonoboUINode     *old,
 		     BonoboWinPrivate *priv)
 {
-	NodeInfo *info = bonobo_ui_xml_get_data (priv->tree, new);
-	NodeInfo *old_info = bonobo_ui_xml_get_data (priv->tree, old);
+	NodeInfo  *info = bonobo_ui_xml_get_data (priv->tree, new);
+	NodeInfo  *old_info = bonobo_ui_xml_get_data (priv->tree, old);
+	GtkWidget *old_widget;
 
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (old_info != NULL);
@@ -946,8 +1009,11 @@ replace_override_fn (GtkObject        *object,
 	info_dump_fn (info);*/
 
 	/* Copy useful stuff across */
+	old_widget = old_info->widget;
+	old_info->widget = NULL;
+
 	info->type = old_info->type;
-	info->widget = old_info->widget;
+	info->widget = old_widget;
 
 	/* Re-stamp the widget */
 	widget_set_node (info->widget, new);
@@ -968,8 +1034,7 @@ prune_widget_info (BonoboWinPrivate *priv,
 	if (!node)
 		return;
 
-	for (l = bonobo_ui_node_children (node);
-             l;
+	for (l = bonobo_ui_node_children (node); l;
              l = bonobo_ui_node_next (l))
 		prune_widget_info (priv, l, save_custom);
 
@@ -983,8 +1048,10 @@ prune_widget_info (BonoboWinPrivate *priv,
 		if (!NODE_IS_ROOT_WIDGET (info) && !save) {
 			GtkWidget *item = get_item_widget (info->widget);
 
-/*			printf ("Destroy widget '%s' '%p'\n",
-			bonobo_ui_xml_make_path (node), item);*/
+#ifdef XML_MERGE_DEBUG
+			printf ("Destroy widget '%s' '%p'\n",
+				bonobo_ui_xml_make_path (node), item);
+#endif
 
 			gtk_widget_destroy (item);
 		} else {
@@ -1006,13 +1073,16 @@ override_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
 	char     *id = node_get_id_or_path (node);
 	NodeInfo *info = bonobo_ui_xml_get_data (priv->tree, node);
 
+#ifdef XML_MERGE_DEBUG
+	fprintf (stderr, "Override '%s'\n", 
+		 bonobo_ui_xml_make_path (node));
+#endif
+
 	prune_widget_info (priv, node, TRUE);
 
 	real_emit_ui_event (priv, info->parent.id, id,
 			    Bonobo_UIComponent_OVERRIDDEN, "");
 
-/*	fprintf (stderr, "XOverride '%s'\n", id); */
-
 	g_free (id);
 }
 
@@ -1022,19 +1092,28 @@ reinstate_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
 	char     *id = node_get_id_or_path (node);
 	NodeInfo *info = bonobo_ui_xml_get_data (priv->tree, node);
 
-/*	fprintf (stderr, "Reinstate '%s'\n", 
-	bonobo_ui_xml_make_path (node));*/
+#ifdef XML_MERGE_DEBUG
+	fprintf (stderr, "Reinstate '%s'\n", 
+		 bonobo_ui_xml_make_path (node));
+#endif
 
 	prune_widget_info (priv, node, TRUE);
 
 	real_emit_ui_event (priv, info->parent.id, id,
 			    Bonobo_UIComponent_REINSTATED, "");
 
-/*	fprintf (stderr, "XReinstate '%s'\n", id);*/
-
 	g_free (id);
 }
 
+static void
+rename_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
+{
+#ifdef XML_MERGE_DEBUG
+	fprintf (stderr, "Rename '%s'\n", 
+		 bonobo_ui_xml_make_path (node));
+#endif
+}
+
 static void
 remove_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
 {
@@ -1044,8 +1123,10 @@ remove_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
 	real_emit_ui_event (priv, info->parent.id, id,
 			    Bonobo_UIComponent_REMOVED, "");
 
-/*	fprintf (stderr, "Remove on '%s'\n",
-	bonobo_ui_xml_make_path (node));*/
+#ifdef XML_MERGE_DEBUG
+	fprintf (stderr, "Remove on '%s'\n",
+		 bonobo_ui_xml_make_path (node));
+#endif
 
 	prune_widget_info (priv, node, FALSE);
 
@@ -1067,8 +1148,6 @@ remove_fn (GtkObject *object, BonoboUINode *node, BonoboWinPrivate *priv)
    gtk_widget_hide (GTK_WIDGET (priv->menu_item)); */
 	}
 
-/*	fprintf (stderr, "XRemove '%s'\n", id);*/
-
 	g_free (id);
 }
 
@@ -1126,6 +1205,12 @@ radio_group_add (BonoboWinPrivate *priv,
 	} else {
 		gtk_radio_menu_item_set_group (
 			menuitem, gtk_radio_menu_item_group (master));
+		
+		/* Since we created this item without a group, it's
+		 * active, but now we are adding it to a group so it
+		 * should not be active.
+		 */
+		GTK_CHECK_MENU_ITEM (menuitem)->active = FALSE;
 	}
 
 	gtk_object_set_data (GTK_OBJECT (menuitem),
@@ -1264,7 +1349,8 @@ put_hint_in_statusbar (GtkWidget *menuitem, BonoboWinPrivate *priv)
 {
 	BonoboUINode *node = widget_get_node (menuitem);
 	BonoboUINode *cmd_node = cmd_get_node (priv, node);
-	char *hint;
+	char *hint, *txt;
+	gboolean err;
 
 	g_return_if_fail (priv != NULL);
 	g_return_if_fail (node != NULL);
@@ -1278,13 +1364,20 @@ put_hint_in_statusbar (GtkWidget *menuitem, BonoboWinPrivate *priv)
 	if (!hint)
 		return;
 
-	if (priv->main_status) {
+	txt = bonobo_ui_util_decode_str (hint, &err);
+	if (err) {
+		g_warning ("Encoding error in tip on '%s', you probably forgot to "
+			   "put an '_' before tip in your xml file",
+			   bonobo_ui_xml_make_path (node));
+	} else if (priv->main_status) {
 		guint id;
 
 		id = gtk_statusbar_get_context_id (priv->main_status,
 						  "BonoboWin:menu-hint");
-		gtk_statusbar_push (priv->main_status, id, hint);
+		gtk_statusbar_push (priv->main_status, id, txt);
 	}
+
+	g_free (txt);
 	bonobo_ui_node_free_string (hint);
 }
 
@@ -1331,17 +1424,22 @@ add_node_fn (BonoboUINode *parent, BonoboUINode *child)
 			g_warning ("TESTME: unused code branch");
                         bonobo_ui_node_insert_before (bonobo_ui_node_children (insert),
                                                       child);
-		} else /* FIXME: we could have 'middle'; is it useful ? */
+		} else
 			bonobo_ui_node_add_child (insert, child);
 		bonobo_ui_node_free_string (pos);
 	} else /* just add to bottom */
 		bonobo_ui_node_add_child (insert, child);
 }
 
+static void
+debug_reparents (BonoboControl *control, const char *str)
+{
+	g_warning ("Control '%s' unmapped\n", str);
+}
+
 static GtkWidget *
 build_control (BonoboWinPrivate *priv,
-	       BonoboUINode     *node,
-	       GtkWidget        *parent)
+	       BonoboUINode     *node)
 {
 	GtkWidget *control = NULL;
 	NodeInfo  *info = bonobo_ui_xml_get_data (priv->tree, node);
@@ -1351,7 +1449,7 @@ build_control (BonoboWinPrivate *priv,
 
 	if (info->widget) {
 		control = info->widget;
-/*		g_assert (info->widget->parent == NULL);*/
+		g_assert (info->widget->parent == NULL);
 	} else if (info->object != CORBA_OBJECT_NIL) {
 
 		control = bonobo_widget_new_control_from_objref
@@ -1360,9 +1458,13 @@ build_control (BonoboWinPrivate *priv,
 		g_return_val_if_fail (control != NULL, NULL);
 		
 		info->type |= CUSTOM_WIDGET;
-		info->widget = control;
 	}
 
+	if (info->widget)
+		gtk_signal_connect (GTK_OBJECT (info->widget), "unmap",
+				    (GtkSignalFunc) debug_reparents,
+				    bonobo_ui_xml_make_path (node));
+
 	do_show_hide (control, node);
 
 /*	fprintf (stderr, "Type on '%s' '%s' is %d widget %p\n",
@@ -1392,7 +1494,7 @@ menu_build_item (BonoboWinPrivate *priv,
 		gtk_widget_set_sensitive (menu_widget, FALSE);
 
 	} else if (bonobo_ui_node_has_name (node, "control")) {
-		GtkWidget *control = build_control (priv, node, parent);
+		GtkWidget *control = build_control (priv, node);
 		if (!control)
 			return NULL;
 		else if (!GTK_IS_MENU_ITEM (control)) {
@@ -1440,6 +1542,8 @@ menu_build_item (BonoboWinPrivate *priv,
 				menu_widget = gtk_pixmap_menu_item_new ();
 			else
 				menu_widget = gtk_menu_item_new ();
+
+			bonobo_ui_node_free_string (txt);
 		}
 			
 		gtk_signal_connect (GTK_OBJECT (menu_widget),
@@ -1458,11 +1562,7 @@ menu_build_item (BonoboWinPrivate *priv,
 		GtkMenu      *menu;
 		GtkWidget    *tearoff;
 		
-		if (parent == NULL) {
-			shell = GTK_MENU_SHELL (priv->menu);
-			g_warning ("FIXME: It would be equitable if this never happened");
-		} else
-			shell = GTK_MENU_SHELL (parent);
+		shell = GTK_MENU_SHELL (parent);
 
 		/* Create the menu shell. */
 		menu = GTK_MENU (gtk_menu_new ());
@@ -1515,7 +1615,8 @@ menu_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	NodeInfo     *info;
 	GtkWidget    *menu_widget;
 	BonoboUINode *cmd_node;
-	char         *sensitive = NULL, *hidden = NULL, *state = NULL, *type, *txt;
+	char         *sensitive = NULL, *hidden = NULL, *state = NULL;
+	char         *type, *txt, *label_attr;
 	static int    warned = 0;
 
 	info = bonobo_ui_xml_get_data (priv->tree, node);
@@ -1527,10 +1628,12 @@ menu_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	    (sensitive = bonobo_ui_node_get_attr (node, "sensitive")) ||
 	    (state     = bonobo_ui_node_get_attr (node, "state"))) {
 		if (cmd_node && !warned++) {
+			char *txt;
 			g_warning ("FIXME: We have an attribute '%s' at '%s' breaking "
 				   "cmd/widget separation, please fix",
 				   hidden?"hidden":((sensitive)?"sensitive":((state)?"state":"error")),
-				   bonobo_ui_xml_make_path (node));
+				   (txt = bonobo_ui_xml_make_path (node)));
+			g_free (txt);
 			if (hidden)
 				set_cmd_attr (priv, cmd_node, "hidden", hidden, FALSE);
 			if (sensitive)
@@ -1584,9 +1687,18 @@ menu_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 		}
 	}
 
-	if ((txt = cmd_get_attr (node, cmd_node, "label"))) {
+	if ((label_attr = cmd_get_attr (node, cmd_node, "label"))) {
 		GtkWidget *label;
 		guint      keyval;
+		gboolean   err;
+
+		txt = bonobo_ui_util_decode_str (label_attr, &err);
+		if (err) {
+			g_warning ("Encoding error in label on '%s', you probably forgot to "
+				   "put an '_' before label in your xml file",
+				   bonobo_ui_xml_make_path (node));
+			return;
+		}
 
 		label = gtk_accel_label_new (txt);
 
@@ -1609,7 +1721,8 @@ menu_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	
 		keyval = gtk_label_parse_uline (GTK_LABEL (label), txt);
 
-		bonobo_ui_node_free_string (txt);
+		bonobo_ui_node_free_string (label_attr);
+		g_free (txt);
 
 		if (keyval != GDK_VoidSymbol) {
 			if (GTK_IS_MENU (parent))
@@ -1689,8 +1802,7 @@ update_menus (BonoboWinPrivate *priv, BonoboUINode *node)
 			      menu_sync_state,
 			      menu_build_item,
 			      menu_build_placeholder);
-	if (wptr)
-		g_warning ("Excess widgets at the end of the container; wierd");
+	check_excess_widgets (priv, wptr);
 
 	if (bonobo_ui_node_parent (node) == priv->tree->root)
 		do_show_hide (info->widget, node);
@@ -1706,7 +1818,7 @@ string_array_contains (char **str_array, const char *match)
 	int i = 0;
 	char *string;
 
-	while ((string = str_array[i++]) != NULL)
+	while ((string = str_array [i++]))
 		if (strcmp (string, match) == 0)
 			return TRUE;
 
@@ -1819,7 +1931,7 @@ toolbar_build_control (BonoboWinPrivate *priv,
 	g_return_val_if_fail (priv != NULL, NULL);
 	g_return_val_if_fail (node != NULL, NULL);
 
-	control = build_control (priv, node, parent);
+	control = build_control (priv, node);
 	if (!control)
 		return NULL;
 
@@ -1925,36 +2037,14 @@ toolbar_build_placeholder (BonoboWinPrivate *priv,
 	return widget;
 }
 
-static char *
-de_accelerate_string (const char *in)
-{
-	char       *ret;
-	const char *a;
-	char       *b;
-
-	g_return_val_if_fail (in != NULL, NULL);
-
-	ret = g_malloc (strlen (in) + 1);
-			
-	a = in;
-	for (b = ret; *a; a++, b++) {
-		*b = *a;
-		if (*b == '_')
-			b--;
-	}
-	*b = '\0';
-
-/*	g_warning ("Testme: made '%s' from '%s'", ret, in); */
-
-	return ret;
-}
-
 static void
 toolbar_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 		    GtkWidget *widget, GtkWidget *parent)
 {
 	char *type, *sensitive = NULL, *state = NULL, *label, *txt, *hidden = NULL;
 	char *min_width;
+	char *behavior;
+	char **behavior_array;
 	GdkPixbuf    *icon_pixbuf;
 	BonoboUINode *cmd_node;
 	static int    warned = 0;
@@ -1983,6 +2073,24 @@ toolbar_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	bonobo_ui_node_free_string (sensitive);
 	bonobo_ui_node_free_string (hidden);
 
+	/* behaviors can apply to controls, so we need to handle them here */
+	if ((behavior = cmd_get_attr (node, cmd_node, "behavior"))) {
+		
+		behavior_array = g_strsplit (behavior, ",", -1);
+		bonobo_ui_node_free_string (behavior);
+
+		bonobo_ui_toolbar_item_set_expandable (
+			BONOBO_UI_TOOLBAR_ITEM (widget),
+			string_array_contains (behavior_array, "expandable"));
+
+		bonobo_ui_toolbar_item_set_pack_end (
+			BONOBO_UI_TOOLBAR_ITEM (widget),
+			string_array_contains (behavior_array, "pack-end"));
+
+		g_strfreev (behavior_array);
+	}
+
+	/* if it's a control, exit now, since the rest isn't relevant */
 	if (bonobo_ui_node_has_name (node, "control"))
 		return;
 
@@ -2000,7 +2108,14 @@ toolbar_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 		}
 
 		if (label) {
-			char *txt = de_accelerate_string (label);
+			gboolean err;
+			char *txt = bonobo_ui_util_decode_str (label, &err);
+			if (err) {
+				g_warning ("Encoding error in label on '%s', you probably forgot to "
+					   "put an '_' before label in your xml file",
+					   bonobo_ui_xml_make_path (node));
+				return;
+			}
 
 			bonobo_ui_toolbar_button_item_set_label (
 				BONOBO_UI_TOOLBAR_BUTTON_ITEM (widget), txt);
@@ -2019,12 +2134,22 @@ toolbar_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	}
 	
 	if ((txt = cmd_get_attr (node, cmd_node, "tip"))) {
+		gboolean err;
+		char *decoded_txt;
 
-		bonobo_ui_toolbar_item_set_tooltip (
-			BONOBO_UI_TOOLBAR_ITEM (widget),
-			bonobo_ui_toolbar_get_tooltips (
-				BONOBO_UI_TOOLBAR (parent)), txt);
+		decoded_txt = bonobo_ui_util_decode_str (txt, &err);
+		if (err) {
+			g_warning ("Encoding error in tip on '%s', you probably forgot to "
+				   "put an '_' before tip in your xml file",
+				   bonobo_ui_xml_make_path (node));
+		} else {
+			bonobo_ui_toolbar_item_set_tooltip (
+				BONOBO_UI_TOOLBAR_ITEM (widget),
+				bonobo_ui_toolbar_get_tooltips (
+					BONOBO_UI_TOOLBAR (parent)), decoded_txt);
+		}
 
+		g_free (decoded_txt);
 		bonobo_ui_node_free_string (txt);
 	}
 }
@@ -2058,15 +2183,14 @@ update_dockitem (BonoboWinPrivate *priv, BonoboUINode *node)
 
 	/* Update the widgets */
 /*	bonobo_win_dump (priv->win, "before build widgets");*/
-	wptr = widgets = gtk_container_children (GTK_CONTAINER (toolbar));
+	wptr = widgets = bonobo_ui_toolbar_get_children (toolbar);
 	pos = 0;
 	sync_generic_widgets (priv, bonobo_ui_node_children (node),
 			      GTK_WIDGET (toolbar), &wptr, &pos,
 			      toolbar_sync_state,
 			      toolbar_build_item,
 			      toolbar_build_placeholder);
-	if (wptr)
-		g_warning ("Excess widgets at the end of the container; wierd");
+	check_excess_widgets (priv, wptr);
 	g_list_free  (widgets);
 /*	bonobo_win_dump (priv->win, "after build widgets");*/
 
@@ -2108,9 +2232,14 @@ update_dockitem (BonoboWinPrivate *priv, BonoboUINode *node)
 	bonobo_ui_toolbar_set_tooltips (toolbar, tooltips);
 #endif
 
-	do_show_hide (GTK_WIDGET (item), node);
 
-	gtk_widget_queue_resize (GTK_WIDGET (item));
+       /*
+	* FIXME: It shouldn't be necessary to explicitly resize the
+	* dock, since resizing a widget is supposed to resize it's parent,
+	* but the dock is not resized correctly on dockitem show / hides.
+	*/
+	if (do_show_hide (GTK_WIDGET (item), node))
+		gtk_widget_queue_resize (GTK_WIDGET (priv->dock));
 
 	bonobo_ui_node_free_string (dockname);
 }
@@ -2196,22 +2325,40 @@ static void
 status_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 		   GtkWidget *widget, GtkWidget *parent)
 {
-	char *name, *txt;
+	char *name;
 		
 	name = bonobo_ui_node_get_attr (node, "name");
 	if (!name)
 		return;
 
 	if (!strcmp (name, "main")) {
+		BonoboUIXmlData *data = bonobo_ui_xml_get_data (priv->tree, node);
+
 		priv->main_status = GTK_STATUSBAR (widget);
 			
-		if ((txt = bonobo_ui_node_get_content (node))) {
+		if (data->id) {
 			guint id;
+			char *txt;
 
-			id = gtk_statusbar_get_context_id (priv->main_status,
-							   "BonoboWin:data");
+			id = gtk_statusbar_get_context_id (
+				priv->main_status, data->id);
+
+			if ((txt = bonobo_ui_node_get_content (node))) {
+				gboolean err;
+				char    *status;
+
+				status = bonobo_ui_util_decode_str (txt, &err);
+
+				if (err)
+					g_warning ("It looks like the status '%s' is not correctly "
+						   "encoded, use bonobo_ui_component_set_status", txt);
+				else
+					gtk_statusbar_push (priv->main_status, id, status);
+
+				g_free (status);
+			} else
+				gtk_statusbar_pop (priv->main_status, id);
 
-			gtk_statusbar_push (priv->main_status, id, txt);
 			bonobo_ui_node_free_string (txt);
 		}
 	}
@@ -2219,12 +2366,18 @@ status_sync_state (BonoboWinPrivate *priv, BonoboUINode *node,
 	bonobo_ui_node_free_string (name);
 }
 
+static void
+main_status_null (GtkObject *dummy, BonoboWinPrivate *priv)
+{
+	priv->main_status = NULL;
+}
+
 static GtkWidget *
 status_build_item (BonoboWinPrivate *priv,
-		    BonoboUINode    *node,
-		    int             *pos,
-		    NodeInfo        *info,
-		    GtkWidget       *parent)
+		   BonoboUINode    *node,
+		   int             *pos,
+		   NodeInfo        *info,
+		   GtkWidget       *parent)
 {
 	char *name;
 	GtkWidget *widget = NULL;
@@ -2237,6 +2390,9 @@ status_build_item (BonoboWinPrivate *priv,
 		widget = gtk_statusbar_new ();
 		priv->main_status = GTK_STATUSBAR (widget);
 
+		gtk_signal_connect (GTK_OBJECT (widget), "destroy",
+				    (GtkSignalFunc) main_status_null, priv);
+
 		/* insert a little padding so text isn't jammed against frame */
 		gtk_misc_set_padding (
 			GTK_MISC (GTK_STATUSBAR (widget)->label),
@@ -2253,43 +2409,75 @@ status_build_item (BonoboWinPrivate *priv,
 			return NULL;
 		}
 
-		widget = build_control (
-			priv, node, GTK_WIDGET (priv->status));
-		if (!widget)
-			return NULL;
+		widget = build_control (priv, node);
 
-		gtk_box_pack_end (GTK_BOX (parent), widget, FALSE, FALSE, 0);
+		if (widget)
+			gtk_box_pack_end (GTK_BOX (parent), widget,
+					  FALSE, FALSE, 0);
 	}
 	bonobo_ui_node_free_string (name);
 
-	gtk_box_reorder_child (priv->status, widget, (*pos)++);
+	if (widget)
+		gtk_box_reorder_child (priv->status, widget, (*pos)++);
 
 	return widget;
 }
 
+static GtkWidget *
+status_build_placeholder (BonoboWinPrivate *priv,
+			  BonoboUINode    *node,
+			  int             *pos,
+			  NodeInfo        *info,
+			  GtkWidget       *parent)
+{
+	GtkWidget *widget;
+
+	g_warning ("TESTME: status bar placeholders");
+
+	widget = bonobo_ui_toolbar_separator_item_new ();
+	gtk_widget_set_sensitive (widget, FALSE);
+
+	gtk_box_pack_end (GTK_BOX (parent), widget,
+			  FALSE, FALSE, 0);
+
+	if (widget)
+		gtk_box_reorder_child (priv->status, widget, (*pos)++);
+
+	return widget;
+}
+
+static GList *
+box_get_children_in_order (GtkBox *box)
+{
+	GList       *ret = NULL;
+	GList       *l;
+
+	g_return_val_if_fail (GTK_IS_BOX (box), NULL);
+
+	for (l = box->children; l; l = l->next) {
+		GtkBoxChild *child = l->data;
+
+		ret = g_list_prepend (ret, child->widget);
+	}
+
+	return g_list_reverse (ret);
+}
+
 static void
 update_status (BonoboWinPrivate *priv, BonoboUINode *node)
 {
 	GtkWidget       *item = GTK_WIDGET (priv->status);
-	BonoboUIXmlData *data;
 	GList *widgets, *wptr;
 	int pos;
 
-	data = bonobo_ui_xml_get_data (priv->tree, node);
-	if (!data->dirty)
-		return;
-
-	priv->main_status = NULL;
-
-	wptr = widgets = gtk_container_children (GTK_CONTAINER (priv->status));
+	wptr = widgets = box_get_children_in_order (GTK_BOX (priv->status));
 	pos = 0;
 	sync_generic_widgets (priv, bonobo_ui_node_children (node),
 			      GTK_WIDGET (priv->status), &wptr, &pos,
 			      status_sync_state,
 			      status_build_item,
-			      toolbar_build_placeholder);
-	if (wptr)
-		g_warning ("Excess widgets at the end of the container; wierd");
+			      status_build_placeholder);
+	check_excess_widgets (priv, wptr);
 	g_list_free  (widgets);
 
 	do_show_hide (item, node);
@@ -2410,6 +2598,8 @@ move_dirt_cmd_to_widget (BonoboWinPrivate *priv)
 				g_warning ("Serious error, cmd without name");
 			else
 				dirty_by_cmd (priv, priv->tree->root, cmd_name);
+
+			bonobo_ui_node_free_string (cmd_name);
 		}
 	}
 }
@@ -2440,6 +2630,8 @@ update_widgets (BonoboWinPrivate *priv)
 	setup_root_widgets (priv);
 	move_dirt_cmd_to_widget (priv);
 
+/*	bonobo_win_dump (priv->win, "before update");*/
+
 	for (node = bonobo_ui_node_children (priv->tree->root); node; node = bonobo_ui_node_next (node)) {
 		if (!bonobo_ui_node_get_name (node))
 			continue;
@@ -2463,6 +2655,8 @@ update_widgets (BonoboWinPrivate *priv)
 	}
 
 	update_commands_state (priv);
+
+/*	bonobo_win_dump (priv->win, "after update");*/
 }
 
 static void
@@ -2795,6 +2989,8 @@ bonobo_win_xml_merge (BonoboWin  *win,
 	g_return_val_if_fail (win->priv != NULL, BONOBO_UI_XML_BAD_PARAM);
 	g_return_val_if_fail (win->priv->tree != NULL, BONOBO_UI_XML_BAD_PARAM);
 
+/*	fprintf (stderr, "Merging :\n%s\n", xml);*/
+
 	node = bonobo_ui_node_from_string (xml);
 	
 	if (!node)
@@ -2953,6 +3149,9 @@ construct_priv (BonoboWin *win)
 	gtk_signal_connect (GTK_OBJECT (priv->tree), "reinstate",
 			    (GtkSignalFunc) reinstate_fn, priv);
 
+	gtk_signal_connect (GTK_OBJECT (priv->tree), "rename",
+			    (GtkSignalFunc) rename_fn, priv);
+
 	gtk_signal_connect (GTK_OBJECT (priv->tree), "remove",
 			    (GtkSignalFunc) remove_fn, priv);
 
