@@ -1,13 +1,54 @@
+/* efstool.c - efs manipulation tool
+
+   Copyright (C) 2000 Maurer IT Systemlsungen KEG
+
+   The Gnome Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The Gnome Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public
+   License along with the Gnome Library; see the file COPYING.LIB.  If not,
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+
+   Author: Dietmar Maurer <dietmar@maurer-it.com>
+
+*/
+
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
 #include <dirent.h>
+#include <stdio.h>
 
-#include "efs.h"
+#include "efs_internal.h"
 
 EFSDir *root;
 gchar path[EFS_MAXPATHLEN];
 
+static gchar*
+get_passwd (const gchar *filename, gboolean new)
+{
+	static gchar passwd[1024];
+
+	
+	if (new) printf("This File (%s) is password protected.\n", filename);
+	printf("Please enter the password: ");
+	       
+	passwd[0] = 0;
+	fgets (passwd, sizeof(passwd)-2, stdin);
+	passwd[sizeof(passwd)] = 0;
+	if (passwd[0] && (passwd[strlen(passwd)-1] == '\n'))
+		passwd[strlen(passwd)-1] = 0;
+	return passwd;
+}
+
 static void
 prompt (void)
 {
@@ -31,39 +72,40 @@ full_path (const char *name)
 static void
 stat_command ()
 {
-	EFSStat *stat;
+	EFSFSStat stat;
 
-	stat = efs_stat(root);
+	if (!efs_fsstat(root, &stat)) {
 
-	printf("EFS Filesystem Summary:\n");
-	printf("Driver:      %s\n",stat->drivername);
-	printf("Block size:  %d\n",stat->blocksize);
-	printf("Name length: %d\n",stat->namelen);
-	printf("Version:     %d\n",stat->version);
-	printf("Blocks:      %d\n",stat->blocks);
-	printf("Free:        %d\n",stat->free);
-	printf("Inodes:      %d\n",stat->inodes);
+		printf("EFS Filesystem Summary:\n");
+		printf("Driver:      %s\n",stat.drivername);
+		printf("Size:        %d\n",stat.size);
+		printf("Name length: %d\n",stat.namelen);
+		printf("Version:     %d\n",stat.version);
+		printf("Free:        %d\n",stat.free);
+	} else {
+		printf("stat command failed\n");
+	}
 }
 
 static void
 ls_command ()
 {
 	EFSDir *dir;
-	EFSDirEntry *de;
+	EFSDirEntry de;
 	const char *name = strtok (NULL, " \t\n");
 
 	if (name) name = strdup(full_path(name));
 	else name = path;
 
-	if (!(dir = efs_dir_open (root, name, 0))) {
+	if (efs_dir_open (&dir, root, name, 0)) {
 		fprintf (stderr, "ls failed!\n");
 		return;
 	}
 
-	while ((de = efs_dir_read(dir))) {
-		if (de->type==EFS_DIR) printf("D ");
+	while (!efs_dir_read(dir, &de)) {
+		if (de.type==EFS_DIR) printf("D ");
 		else printf("  ");
-		printf("%s\n",de->name);
+		printf("%s\n",de.name);
 	}
 
 	efs_dir_close (dir);
@@ -125,7 +167,7 @@ get_command ()
 
 	name = strdup (full_path (name));
 
-	if (!(fsrc = efs_file_open (root, name, EFS_READ))) {
+	if (efs_file_open (&fsrc, root, name, EFS_READ)) {
 		fprintf (stderr,"efs open failed (%s)\n", name);
 		return;
 	}
@@ -143,13 +185,11 @@ get_command ()
 
 	while (1) {
 		char   buffer [1024];
-		gint32 len;
+		gint32 br;
 
-		len = efs_file_read (fsrc, buffer, 1024);
-		if (len <= 0)
-			break;
+		if (efs_file_read (fsrc, buffer, 1024, &br)) break;
 
-		fwrite (buffer, len, 1, fdest);
+		fwrite (buffer, br, 1, fdest);
 	}
 
 	efs_file_close (fsrc);
@@ -171,7 +211,7 @@ mkdir_command ()
 
 	name = strdup(full_path(name));
 
-	if (!(dir = efs_dir_open(root, name, EFS_CREATE|EFS_EXCL))) {
+	if (efs_dir_open(&dir, root, name, EFS_CREATE|EFS_EXCL)) {
 		fprintf(stderr,"mkdir failed (%s)\n", name);
 	} else efs_dir_close (dir);
 }
@@ -210,7 +250,7 @@ cd_command ()
 		strcat (path, to);
 	}
 
-	if (!(dir = efs_dir_open(root, path, 0))) {
+	if (efs_dir_open(&dir, root, path, 0)) {
 		fprintf(stderr,"No such directory\n");
 		strcpy (path, op);
 		return;
@@ -245,8 +285,6 @@ add_file_or_dir (gchar *name)
 	DIR *d;
 	struct dirent *de;
 
-	printf("ADD %s\n", name);
-
 	if (lstat(name,&sb) == -1) {
 		printf("add %s failed!\n", name);
 		return;
@@ -254,7 +292,7 @@ add_file_or_dir (gchar *name)
 
 	if (S_ISDIR(sb.st_mode)) {
 		if (!(d = opendir (name))) return;
-		if (!(dir = efs_dir_open(root, name, EFS_CREATE|EFS_EXCL))) 
+		if (efs_dir_open(&dir, root, name, EFS_CREATE|EFS_EXCL)) 
 			return;
 		efs_dir_close (dir);
 
@@ -269,7 +307,7 @@ add_file_or_dir (gchar *name)
 		return;
 	}
 	if (S_ISREG(sb.st_mode)) {
-		if (!(file=efs_file_open (root, name, EFS_CREATE|EFS_EXCL))) 
+		if (efs_file_open (&file, root, name, EFS_CREATE|EFS_EXCL)) 
 			return;
 		if ((fd = open (name, O_RDONLY)) == -1) return;
 
@@ -283,27 +321,28 @@ add_file_or_dir (gchar *name)
 		return;
 	}
 
-	printf("add %s failed!\n", name);
+	printf("add %s failed (not a file)!\n", name);
 }
 
 static void
 list_contents (EFSDir *dir)
 {
 	EFSDir *d;
-	EFSDirEntry *de;
+	EFSDirEntry de;
 
 	efs_dir_seek (dir, 0);
-	while ((de = efs_dir_read (dir))) {
-		printf("%s\n", de->name);
+	while (efs_dir_read (dir, &de) > 0) {
+		printf("%s\n", de.name);
 	}
 
 	efs_dir_seek (dir, 0);
-	while ((de = efs_dir_read (dir))) {
-		if (de->type == EFS_DIR) {
-			d = efs_dir_open (dir, de->name, EFS_READ);
-			printf("\n%s:\n",de->name);
-			list_contents(d);
-			efs_dir_close(d);
+	while (efs_dir_read (dir, &de) > 0) {
+		if (de.type == EFS_DIR) {
+			if (!efs_dir_open (&d, dir, de.name, EFS_READ)) {
+				printf("\n%s:\n",de.name);
+				list_contents(d);
+				efs_dir_close(d);
+			}
 		}
 	}
 }
@@ -328,8 +367,7 @@ main(int argc, char *argv[])
 	}
 
 	if (argc == 2) { /* interactive mode */
-
-		if (!(root = efs_open(argv[1], EFS_RDWR, 0644))) {
+		if (efs_open_cb(&root,argv[1],EFS_RDWR,0644,get_passwd)) {
 			fprintf (stderr, "efs_open failed\n");
 			exit (1);
 		}
@@ -378,8 +416,8 @@ main(int argc, char *argv[])
 
 		if (!strcmp(argv[1], "c")&&(argc>=4)) {
 
-			if (!(root = efs_open(argv[2], 
-					      EFS_CREATE|EFS_EXCL, 0644))) {
+			if (efs_open(&root, argv[2], EFS_CREATE|EFS_EXCL, 
+				     0644, NULL)) {
 				fprintf (stderr, "efs_open failed\n");
 				exit (1);
 			}
@@ -394,7 +432,7 @@ main(int argc, char *argv[])
 		}
 		if (!strcmp (argv[1], "t")&&(argc==3)) {
 
-			if (!(root = efs_open(argv[2], EFS_READ, 0))) {
+			if (efs_open(&root, argv[2], EFS_READ, 0, NULL)) {
 				fprintf (stderr, "efs_open failed\n");
 				exit (1);
 			}
