@@ -3,11 +3,7 @@
 #include <gnome.h>
 #include <bonobo/bonobo.h>
 
-#if USING_OAF
 #include <liboaf/liboaf.h>
-#else
-#include <libgnorba/gnorba.h>
-#endif
 
 #include "bonobo-moniker-file.h"
 #include "bonobo-moniker-item.h"
@@ -19,7 +15,7 @@ static int running_objects = 0;
 static BonoboGenericFactory *std_factory = NULL;
 
 static void
-std_destroy_cb (BonoboEmbeddable *embeddable, gpointer dummy)
+std_destroy_cb (BonoboMoniker *moniker, gpointer dummy)
 {
 	running_objects--;
 
@@ -112,13 +108,9 @@ main (int argc, char *argv [])
 
 	CORBA_exception_init (&ev);
 
-#ifdef USING_OAF
         gnome_init_with_popt_table ("std-moniker", "0.0", argc, argv,
 				    oaf_popt_options, 0, NULL); 
 	orb = oaf_init (argc, argv);
-#else
-	g_error ("Monikers require oaf");
-#endif
 
 	if (bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
 		g_error (_("I could not initialize Bonobo"));
